// File: cube.cxx
// An OpenGL program to draw a three-dimensional cube
// Written by Grant Macklem
// Last Updated: August 6, 1999


#include <stdlib.h>    // Provides exit on windows systems
#include <GL/glut.h>   // Provides OpenGL graphics routines
#include <iostream>

using namespace std;


// Prototypes
GLfloat clamp(GLfloat& angle);

// Global Variables
GLfloat xrot = 30.0, yrot = 45.0, zrot = 0.0;


void display()
{
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glTranslatef(0.0, 0.0, -30.0);
    glRotatef(clamp(xrot), 1.0, 0.0, 0.0);
    glRotatef(clamp(yrot), 0.0, 1.0, 0.0);
    glRotatef(clamp(zrot), 0.0, 0.0, 1.0);

    glBegin(GL_QUADS);
    
    // Top
    glColor3f(1.0, 0.0, 0.0);     // Red
    glVertex3f(10.0, 10.0, 10.0);
    glVertex3f(10.0, 10.0, -10.0);
    glVertex3f(-10.0, 10.0, -10.0);
    glVertex3f(-10.0, 10.0, 10.0);

    // Right Side
    glColor3f(0.0, 1.0, 0.0);      // Green
    glVertex3f(10.0, 10.0, 10.0);
    glVertex3f(10.0, -10.0, 10.0);
    glVertex3f(10.0, -10.0, -10.0);
    glVertex3f(10.0, 10.0, -10.0);

    // Back Side
    glColor3f(0.0, 0.0, 1.0);      // Blue
    glVertex3f(10.0, -10.0, -10.0);
    glVertex3f(-10.0, -10.0, -10.0);
    glVertex3f(-10.0, 10.0, -10.0);
    glVertex3f(10.0, 10.0, -10.0);

    // Left Side
    glColor3f(1.0, 1.0, 0.0);      // Yellow
    glVertex3f(-10.0, 10.0, -10.0);
    glVertex3f(-10.0, -10.0, -10.0);
    glVertex3f(-10.0, -10.0, 10.0);
    glVertex3f(-10.0, 10.0, 10.0);

    // Front Side
    glColor3f(0.0, 1.0, 1.0);      // Cyan
    glVertex3f(10.0, -10.0, 10.0);
    glVertex3f(10.0, 10.0, 10.0);
    glVertex3f(-10.0, 10.0, 10.0);
    glVertex3f(-10.0, -10.0, 10.0);
   
    // Bottom
    glColor3f(1.0, 0.0, 1.0);      // Magenta
    glVertex3f(-10.0, -10.0, 10.0);
    glVertex3f(-10.0, -10.0, -10.0);
    glVertex3f(10.0, -10.0, -10.0);
    glVertex3f(10.0, -10.0, 10.0);

    glEnd();
    
    glutSwapBuffers();
    
}


GLfloat clamp(GLfloat& angle)
{

/* Must use greater than or equal to because on systems that only have
 * software rendering available, the keyboard buffer may get filled with
 * commands to rotate while the display is rendering the last rotated frame
 * thereby increasing the angle past 360 degrees.
 */
    
    if (angle >= 360.0)
    angle = 0;
    return angle;
}


void reshape(int w, int h)
{
//    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glViewport(0, 0, (GLint) w, (GLint) h);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    glFrustum(-5.0, 5.0, -5.0, 5.0, 5.0, 60.0);
//    glOrtho(-5.0, 5.0, -5.0, 5.0, 0.0, 100.0);

//    gluPerspective(90, (GLint) w / (GLint) h, 1, 9999);
    
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(0.0, 0.0, -30.0);
    
    glutPostRedisplay();

}


void keyboard(unsigned char key, int x, int y)
{
    switch (key)
    {
    case '/':
        zrot += 5.0;
        break;
    case '\\':
        zrot -= 5.0;
        break;
    case 'q': case 'Q': case (char) 27: /* Escape */
        exit(0);
        break;
    default:
        return;
    }

    glutPostRedisplay();
}


void special(int key, int x, int y)
{
    switch(key)
    {
    case GLUT_KEY_UP:
        xrot += 5.0;
        break;
    case GLUT_KEY_DOWN:
        xrot -= 5.0;
        break;
    case GLUT_KEY_LEFT:
        yrot += 5.0;
        break;
    case GLUT_KEY_RIGHT:
        yrot -= 5.0;
        break;
    default:
        return;
    }

    glutPostRedisplay();
}


void init()
{

    glClearColor(0.0, 0.0, 0.0, 0.0);
    glClearDepth(1.0);
    glShadeModel(GL_FLAT);
    glEnable (GL_DEPTH_TEST);
    glPolygonMode (GL_FRONT, GL_FILL);
//    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

//   For cool, front surface removal, uncomment these two lines
//    glPolygonMode(GL_BACK, GL_LINE);
//    glFrontFace(GL_CW);
    
}


int main(int argc, char** argv)
{
    cout << "ARGC: " << argc << endl;
    glutInit(&argc, argv);
    glutInitDisplayMode (GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB);
    glutInitWindowSize(400, 400);
    glutCreateWindow("Cube");

    glutDisplayFunc(display);
    glutReshapeFunc(reshape);
    glutKeyboardFunc(keyboard);
    glutSpecialFunc(special);

    init();
    glutMainLoop();
    return 0;
}
